﻿import { toPixel, toLngLat } from '../utils/convert-helper.js';
import eventHelper from '../utils/event-helper.js'
export default class MAP {
    constructor(config) {
            if (!config) throw new Error("config 不能为空");
            this.mid = config.mid;
            this.mapOption = config.mapOption;
            this.plugins = config.plugins;
            this.events = config.events;
            this.onceEvents = config.onceEvents;
            let mapObj = null;
            this.load();
        }
        /**
         * 初始化
         */
    load() {
        this.mapObj = new AMap.Map(this.mid, this.mapOption);
        this.registerEvents();
        this.addMapControls();
        if (this.events && this.events.init) {
            this.events.init(this.mapObj); //初始化
        }
    }

    /**
     * 
     * @param {添加地图控件} plugins 
     */
    addMapControls() {
        if (!this.plugins || !this.plugins.length) return;
        this.plugins.forEach(function(_plugin) {
            let realPlugin = this.convertAMapPluginProps(_plugin);
            this.plugins[realPlugin.pName] = new AMap[realPlugin.pName](realPlugin);
            this.mapObj.addControl(this.plugins[realPlugin.pName]); //添加控件
            if (_plugin.events) {
                if (realPlugin.events.init) {
                    realPlugin.events.init(this.plugins[realPlugin.pName]);
                }

                for (let k in _plugin.events) {
                    let v = _plugin.events[k];
                    if (k === 'init') continue;

                    eventHelper.addListener(this.plugins[realPlugin.pName], k, v);
                    // AMap.event.addListener(this.plugins[realPlugin.pName], k, v);
                }
            }
        }, this);
    }

    /**
     * 
     * @param {地图控件参数转换} _plugin 
     */
    convertAMapPluginProps(_plugin) {
        if (typeof _plugin != 'object' && !_plugin.pName)
            throw new Error("_plugin.pName 是必填的");
        switch (_plugin.pName) {
            case 'AMap.ToolBar':
                if (_plugin.offset && _plugin.offset instanceof Array) {
                    _plugin.offset = toPixel(_plugin.offset);
                }
                break;
            case 'AMap.Scale':
                if (_plugin.offset && _plugin.offset instanceof Array) {
                    _plugin.offset = toPixel(_plugin.offset);
                }
                break;
            case 'ControlBar':

                break;

            case "OverView":
                break;

            case "MapType":

                break;
        }
        return _plugin;
    }

    /***
     * 注册事件
     */
    registerEvents() {
        if (this.events) {
            for (let eventName in this.events) {
                eventHelper.addListener(this.mapObj, eventName, this.events[eventName])
            }
        }
        if (this.onceEvents) {
            for (let eventName in this.onceEvents) {
                eventHelper.addListener(this.mapObj, eventName, this.onceEvents[eventName])
            }
        }
    }

    /**
     * 取消事件
     */
    unregisterEvents() {
        eventHelper.clearListeners(this.mapObj);
    }

    /**
     * 
     * @param {级别} zoom 
     * @param {中心点} center 
     */
    setZoomAndCenter(zoom, center) {
        this.mapObj.setZoomAndCenter(zoom, center);
    }

    /**
     *自适应地图
     * @param {经纬度数组} xy
     */
    setFitView(xy) {
        var bounds = this.mapObj.getBounds();
        var lnglat = toLngLat(xy);
        if (!bounds.contains(lnglat)) {
            this.mapObj.setCenter(lnglat);
        }
    }

    /**
     * 删除地图上所有的覆盖物
     */
    clearMap() {
        this.mapObj.clearMap();
    }
}