﻿
import { lngLatTo, toLngLat, toPixel } from '../utils/convert-helper.js';
import eventHelper from '../utils/event-helper.js'
export default class AMapMarkerClusterer {
    constructor(config) {
        this.mapObj = config.map;
        this.styles = config.styles;
        this.overyMarks = new Map(); //缓存标记信息
        this.cluster = null;
        this.load();
    }
    load() {
        this.cluster = new AMap.MarkerClusterer(this.mapObj, [], this.styles);
    }
    /**
     * Marker转换标记对象
     */
    converMarkerProp(markers) {
        //  let mdata = [];
        markers.forEach(function (mark) {
            let _marker = mark.markerOpt;
            _marker.map = this.mapObj;
            if (_marker.position && _marker.position instanceof Array) {
                _marker.position = toLngLat(_marker.position);
            }
            if (_marker.offset && _marker.offset instanceof Array) {
                _marker.offset = toPixel(_marker.offset);
            }
            if (_marker.label && _marker.label instanceof Object) {
                _marker.label.offset = toPixel(_marker.label.offset);
            }
        }, this);
        return markers;
    }

    /**
     * 注册事件
     * @param {对象} markers 
     */
    registerEvents(markers) {
        let marks = [];
        markers.forEach(function (mark) {
            let markinfo = new AMap.Marker(mark.markerOpt);
            for (let k in mark.events) {
                let v = mark.events[k];
                if (k === 'init') {
                    v(markinfo);
                } else {
                    eventHelper.addListener(markinfo, k, v);
                }
            }
            marks.push(markinfo);
        })
        return marks;
    }
    /**
     * 
     * @param {移除监听} event 
     */
    removeListener(mark) {
        console.log("还没实现这个功能");
    }
    /**
     * 
     * @param {更改标记} marker 
     */
    updataMarker(marker) {

        let _mark = this.overyMarks.get(marker.key);
        if (_mark) {
            if (marker.position && marker.position instanceof Array) {
                _mark.setPosition(toLngLat(marker.position));
            }
            // console.log('updataMark' + marker.label.content);
            if (marker.label && marker.label instanceof Object) {
                _mark.setLabel({
                    content: marker.label.content,
                    //offset = marker.label.offset ? toPixel(_marker.label.offset) : _mark.label.offset
                })
            }
            // debugger
            _mark.setIcon(marker.icon);
            _mark.setExtData(marker.extData);
            this.overyMarks.set(marker.key, _mark);
        }
    }
    /**
     * 
     * @param {批量修改标记} markers 
     */
    updataMarkers(markers) {
        markers.forEach(function (el) {
            this.updataMarker(el);
        }, this);
    }
    /**
     * 
     * @param {添加一个需进行聚合的点标记} marker 
     */
    addMarker(marker) {
        if (typeof this.cluster != 'object' && !this.cluster)
            throw new Error("必须先初始化聚合实例");
        let _realMarker = this.converMarkerProp([marker]);


        let _markers = this.registerEvents(_realMarker)

        this.overyMarks.set(marker.markerOpt.key, _markers[0]);
        
        this.cluster.addMarker(_markers[0]);
        //this.mapObj.setZoomAndCenter(14, [116.205467, 39.907761]);
    }
    /**
     * 
     * @param {添加一组需进行聚合的点标记} markers 
     */
    addMarkers(markers) {

        if (typeof this.cluster != 'object' && !this.cluster)
            throw new Error("必须先初始化聚合实例");

        let _realMarker = this.converMarkerProp(markers);
        let _marks = this.registerEvents(_realMarker);

        _marks.forEach(function (element) {

            this.overyMarks.set(element.G.key, element);
        }, this);

        this.cluster.addMarkers(_marks);
    }
    /**
     * 
     * @param {删除一个聚合的点标记} marker 
     */
    removeMarker(key) {
        if (typeof this.cluster != 'object' && !this.cluster)
            throw new Error("必须先初始化聚合实例");

        var _mark = this.overyMarks.get(key);
        this.cluster.removeMarker(_mark);
        this.overyMarks.delete(key);
    }
    /**
     * 
     * @param {删除一组聚合的点标记} markers 
     */
    removeMarkers(keys) {
        if (typeof this.cluster != 'object' && !this.cluster)
            throw new Error("必须先初始化聚合实例");

        var _marks = [];
        keys.forEach(function (el) {
            let _mark = this.overyMarks.get(el);
            _marks.push(_mark);
        })

        this.cluster.removeMarkers(_marks);

        keys.forEach(function (ele) {
            this.overyMarks.delete(ele);
        })

    }
    /**
     * 获取聚合点的总数量
     */
    getClustersCount() {
        return this.cluster.getClustersCount();
    }
    /**
     * 从地图上彻底清除所有聚合点标记
     */
    clearMarkers() {
        if (typeof this.cluster != 'object' && !this.cluster)
            throw new Error("必须先初始化聚合实例");
        this.cluster.clearMarkers();
        this.overyMarks.clear();
    }
    /**
     * 获取该点聚合中的点标记集合
     */
    getMarkers() {
        if (typeof this.cluster != 'object' && !this.cluster)
            throw new Error("必须先初始化聚合实例");
        return this.cluster.getMarkers();
    }

}