﻿import { toLngLat } from '../utils/convert-helper.js';
export default class trackInfo {
    /**
     * 
     * @param {构造参数} config 
     */
    constructor(config) {
        this.config = config;
        this.pathSimplifierIns = null;
        this.navg0 = null;
        this.init();
    }

    //初始化
    init() {
        let $this = this;
        //加载PathSimplifier，loadUI的路径参数为模块名中 'ui/' 之后的部分 
        AMapUI.load(['ui/misc/PathSimplifier'], function(PathSimplifier) {
            if (!PathSimplifier.supportCanvas) {
                alert('当前环境不支持 Canvas！');
                return;
            }
            $this.pathSimplifierIns = new PathSimplifier($this.config);
        });
    }

    /**
     * 为轨迹展示组件 PathSimplifier 绑定事件
     * @param {*} evt 事件名称
     * @param {*} callback  回调函数 
     */
    onPathSimplifierEvt(evt, callback) {
        if (this.pathSimplifierIns != null) {//throw new Error('[为轨迹展示组件 PathSimplifier 绑定事件]轨迹展示组件PathSimplifier为空，请先初始化巡航对象!');
            this.pathSimplifierIns.on(evt, callback);
        }
    }

    /**
     * 设定数据源数组，并触发重新绘制。data为空时将清除显示内容。 
     * @param {数组对象} data 
     */
    setData(data) {
        if (this.pathSimplifierIns != null) {//throw new Error('[设定数据源数组]轨迹展示组件PathSimplifier为空，请先初始化巡航对象!');
            if (data instanceof Array) {
                this.pathSimplifierIns.setData(data);
            } else {
                alert('传入数据必须是数组');
            }
        }
    }

    /**
     * 创建一个轨迹巡航器。
     * @param {关联的轨迹索引} pathIndex 
     * @param {巡航器的配置选项，详见下方轨迹巡航器部分} options 
     */
    createPathNavigator(pathIndex, options) {
        if (this.pathSimplifierIns != null) {//throw new Error('[创建一个轨迹巡航器]轨迹展示组件PathSimplifier为空，请先初始化巡航对象!');

            if (options instanceof Object) {
                //释放之前创建的对象
                if (this.navg0 != null) {
                    this.navg0.destroy();
                    this.navg0 = null;
                }

                //关联第pathIndex条轨迹
                this.navg0 = this.pathSimplifierIns.createPathNavigator(pathIndex, options);
            } else {
                alert('第二个参数必须是对象');
            }
        }
    }

    /**
     * 释放之前创建的巡航器实例
     */
    disposeNavigator() {
        //释放之前创建的对象
        if (this.navg0 != null) {
            this.navg0.destroy();
            this.navg0 = null;
        }
    }

    /**
     * 开始路径巡航。
     * @param { pointIndex用于指定巡航的起始节点索引，默认为节点索引范围（range）的最小值 如果是浮点类型，则整数部分表示起始点索引，小数部分表示起始点和下一节点间的比例位置。比如7.5就表示7~8的中间位置} pointIndex 
     */
    navigatorStart(pointIndex) {
        if (this.navg0 != null) {//throw new Error('[开始路径巡航]巡航器组件为空，请先创建轨迹巡航器!');

            if (pointIndex) {
                this.navg0.start(pointIndex);
            } else {
                this.navg0.start();
            }
        }
    }

    /**
     * 暂停巡航
     */
    navigatorPaus() {
        if (this.navg0 != null) {//throw new Error('[暂停巡航]巡航器组件为空，请先创建轨迹巡航器!');
            this.navg0.pause();
        }
    }

    /**
     * 恢复巡航
     */
    navigatorResume() {
        if (this.navg0 != null) {// throw new Error('[恢复巡航]巡航器组件为空，请先创建轨迹巡航器!');
            this.navg0.resume();
        }
    }

    /**
     * 停止巡航，同时清除已经过路径（这一点不同于pause）
     */
    navigatorStop() {
        if (this.navg0 != null) {// throw new Error('[停止巡航]巡航器组件为空，请先创建轨迹巡航器!');
            this.navg0.stop();
        }
    }

    /**
     * 销毁巡航器。巡航作为动画过程是非常耗费性能的，因此不再需要时应及时销毁
     */
    navigatorDestroy() {
        if (this.navg0 != null) {// throw new Error('[销毁巡航器]巡航器组件为空，请先创建轨迹巡航器!');
            this.navg0.destroy();
        }
    }

    /**
     * 为巡航绑定事件处理函数
     * @param {参数} eventName 事件名称: start(开始巡航)、pause(暂停巡航)、move(移动)、stop(停止)
     * @param {方法} callback 
     */
    navigatorEvent(eventName, callback) {
        if (this.navg0 != null) {// throw new Error('[为巡航绑定事件处理函数]巡航器组件为空，请先创建轨迹巡航器!');
            if (callback instanceof Function)
                this.navg0.on(eventName, callback);
            else {
                alert('传入参数必须是方法');
            }
        }
    }

    /**
     * 设定巡航器的速度，单位千米/小时
     * @param {*} speed 
     */
    navigatorSetSpeed(speed) {
        if (this.navg0 != null) {//return; //throw new Error('[设定巡航器的速度]巡航器组件为空，请先创建轨迹巡航器!');
            this.navg0.setSpeed(speed);
        }
    }

    /**
     * 将巡航器移动到指定的节点索引位置
     * @param {*} idx 
     * @param {*} tail 
     */
    navigatorMoveToPoint(idx, tail) {
        if (this.navg0 == null) return;
        this.navg0.moveToPoint(idx, tail);
    }

    /**
     * 返回当前所处的索引位置。返回属性为:
    {
        idx: {number}， 节点索引
        tail: {number}，至下一个节点的比例位置
    }
    比如{idx: 1, tail:0.5}就表示当前处于节点1到节点2的中间位置
     */
    navigatorGetCursor() {
        if (this.navg0 == null) return null;
        return this.navg0.getCursor();
    }
}